import 'proxy-polyfill';
import 'es6-object-assign/auto';
import 'es6-symbol/implement';
import { getProxySDK } from '../../internal/src/client/proxy';
import { _configuration } from '../../internal/src/client/symbols';
import { IInitOptions } from '../../internal/src/middleware/types';

const getSrc = (org: string) => {
  let origin = 'https://api.commandbar.com';
  const params = [];

  const lc = localStorage.getItem('commandbar.lc');
  if (lc && lc.includes('local')) {
    // for debugging purposes
    origin = 'http://localhost:8000';
  }

  let src = origin + '/latest/' + org;

  if (lc) {
    params.push('lc=' + lc);
  }

  const version = 2;
  params.push('version=' + version);

  src = src + '?' + params.join('&');
  return src;
};

const loadLatest = (org: string) => {
  if (document.readyState !== 'complete') {
    // NOTE: `once` is not supported by IE11 and this will cause a (minor!) memory leak.
    window.addEventListener('load', loadLatest.bind(null, org), { capture: false, once: true });
    return;
  }

  const src = getSrc(org);

  const el = document.createElement('script');
  el.type = 'text/javascript';
  el.async = true;
  el.src = src;
  document.head.appendChild(el);
};

const DEFAULT_OPTS = { debug: false };

/**
 * Initializes CommandBar and injects the scripts required to load it into the <head> of the document. After calling
 * this function it will be possible to access window.CommandBar and call the SDK methods, even if the scripts have not
 * finished loading yet.
 *
 * @param {string} org Your organization ID, which can be found [here](https://commandbar.com/docs/dev/installation).
 * @param {IInitOptions} [opts] Additional options to use for initialization: `debug` will enable additional logging, `environment` specifies the environment to load config from
 * output.
 */
const init = (org: string, opts: IInitOptions = DEFAULT_OPTS) => {
  const config = { ...DEFAULT_OPTS, ...opts };

  if (!org) {
    console.error('No org specified for CommandBar.init');
    return;
  }

  // Do nothing if CommandBar SDK has already been initialized
  if (!!getProxySDK()[_configuration]?.uuid) {
    console.warn('CommandBar init was called more than once. Skipping the redundant initialization...');
    return;
  }

  config.debug && console.log('CommandBar init...', { environment: opts.environment, version: opts.version });

  getProxySDK()[_configuration] = {
    uuid: org,
    environment: opts.environment,
    version: opts.version,
    config: opts.config,
  };
  loadLatest(org);
};

export default init;
